import pandas as pd
import numpy as np
import matplotlib
from matplotlib import pyplot as plt
import pmdarima as pm

def aggregate_period(df, period, date, target): 
    if period == 'w':
        df[date] = df[date].dt.week
        g = df.groupby(date).agg({target: ['sum']})
        g.columns = [target]
        g.reset_index(level=0, inplace=True)
    else:
        per = df[date].dt.to_period(period)
        g = df.groupby(per).agg({target: ['sum']})
        g.columns = [target]
        g.reset_index(level=0, inplace=True)

        # make sure it is datetime format
        g[date] = g[date].astype(str)
        g[date] = pd.to_datetime(g[date],errors='coerce')
    return g


def extract_period(df, date, target, period):
    df2 = df[[date , target]]
    # drop NA
    df2 = df2.dropna()
    
    # coerce datetime
    df2[date] = pd.to_datetime(df2[date],errors='coerce')
    
    # check if target is numeric
    if pd.api.types.is_numeric_dtype(df2[target]):
        pass
    else:
        # raise error
        raise Exception("the target columns must be numerical type, please check your data")
    
    # aggregate by period and sum target
    df3 = aggregate_period(df2, period, date, target)
    
    ## check if date is equal interval
    df4 = df3.copy()
    df4['check_date'] = df4[date].shift(periods=1)
    df4['check_date2'] = df4[date].dt.to_period(period).view(dtype='int64') - df4['check_date'].dt.to_period(period).view(dtype='int64')
    df4 = df4.dropna()
    date_dict = {'y':['year', 'years'], 'm':['month','months'], 'w':['week','weeks'], 'd': ['day', 'days']}

    if len(np.unique(df4['check_date2']))==1:
        print(f"The date interval for each data points is the same, of '{date_dict[period][0]}' interval")
    else:
        print("All values are not exactly equal, not ideal for forcasting. Suggest to change to a broader period or treat the date first'")
    
    return df3

def calibrate_airma(df3, target):
    print("Calibrating best ARIMA")
    best_arima = pm.auto_arima(df3[target], error_action ='ignore')
    
    print(f"Optimised: {best_arima}")
    
    best_arima_model_name = f'ARIMA(p={best_arima.order[0]}, d={best_arima.order[1]}, q={best_arima.order[2]})'
    
    return best_arima, best_arima_model_name

# FUNCTION
def get_forecast_values(df, date, best_arima, N, target, period,show_info=False):
    
    # to get last date of actual
    last_date = df.reset_index().at[len(df)-1, date]
    
    # to get forcast values and with 95% confidence interval
    forecast, conf_int = best_arima.predict(n_periods=N, typ='levels' ,return_conf_int=True,alpha=0.05)
    
    # check if forcasted period is it valid?
    if N > int(0.20*len(df)):
        print(f"The number of period to be forcasted, N, is more than one quarter of the data points, change it to {int(0.20*len(df))} or lower?")
    
    # if period range is in days
    if period == 'd':
        
        # to get start year of forcasting
        for_date = pd.to_datetime(last_date) + pd.DateOffset(days=1)
        
        # to get a range of forcasting date (in day)
        index = pd.date_range(start=for_date, periods=N, freq='D')
        
        # to make the forecast values to a dataframe
        df_forecast = pd.DataFrame({target:forecast},index=index)

        # to make confidence into interval
        confidence = pd.DataFrame(conf_int, columns=['lower', 'upper'],index=index)
        
        # make sure index is date time
        df_forecast.index = pd.to_datetime(df_forecast.index,errors='coerce')
        confidence.index = pd.to_datetime(confidence.index,errors='coerce')
    
    elif period == 'y':
        
        # to get start year of forcasting
        for_date = pd.to_datetime(last_date) + pd.DateOffset(years=1)
        
        # to get a range of forcasting date (in beginning year)
        index = pd.date_range(start=for_date, periods=N,freq='AS')
        
        # to make the forecast values to a dataframe
        df_forecast = pd.DataFrame({target:forecast},index=index)
        
        # to make confidence into interval
        confidence = pd.DataFrame(conf_int, columns=['lower', 'upper'],index=index)
        
        # make sure index is date time
        df_forecast.index = pd.to_datetime(df_forecast.index,errors='coerce')
        confidence.index = pd.to_datetime(confidence.index,errors='coerce')
        
    elif period == 'm':
        
        # to get start month of forcasting
        for_date = pd.to_datetime(last_date) + pd.DateOffset(months=1)
        
        # to get a range of forcasting date (in beginning month)
        index = pd.date_range(start=for_date, periods=N,freq='MS')
        
        # to make the forecast values to a dataframe
        df_forecast = pd.DataFrame({target:forecast},index=index)
        
        # to make confidence into interval
        confidence = pd.DataFrame(conf_int, columns=['lower', 'upper'],index=index)
        
        # make sure index is date time
        df_forecast.index = pd.to_datetime(df_forecast.index,errors='coerce')
        confidence.index = pd.to_datetime(confidence.index,errors='coerce')
        
    elif period == 'w':
        
        # to get start week of forcasting
        for_date = int(last_date) + 1
        
        # to get a range of forcasting date (in week)
        index = np.arange(for_date, for_date+N)
        
        # to make the forecast values to a dataframe
        df_forecast = pd.DataFrame({target:forecast},index=index)
        
        # to make confidence into interval
        confidence = pd.DataFrame(conf_int, columns=['lower', 'upper'],index=index)
        
    return df_forecast, confidence, last_date, N

def concat_result(df3, forecast, date):
    actual = df3.set_index(date)
    frames = [actual, forecast]
    df_concat = pd.concat(frames)
    
    return df_concat

def vis(target,df_concat,last_date,confidence, show_model_name,best_arima_model_name,my_title = True, y_label_name = True, x_label_name = True,\
        color_past='dodgerblue', color_forecast='crimson',linestyle_past = 'solid', linestyle_forecast='--', \
        show_confidence=True, numElems=10, matplotlib_style = 'seaborn', \
        legend_position=2, fig_length=13.5, fig_height=6.5, legend_font_size=14, scientific_annotation=True, \
       zoom_last_n = True):
    
    # to set theme
    matplotlib.style.use(matplotlib_style)
    
    # to receive values
    t = df_concat.index[-zoom_last_n:]    
    past = t <= last_date
    future = t >= last_date
    target_points = df_concat[target].values[-zoom_last_n:]
    lower_confidence = confidence.lower.values[-zoom_last_n:]
    upper_confidence = confidence.upper.values[-zoom_last_n:]
    
    # setup figures
    fig, ax=plt.subplots()
    fig = matplotlib.pyplot.gcf()
    fig.set_size_inches(fig_length, fig_height)

    # To plot past actual data
    ax.plot(t[past], target_points[past], label='actual', color=color_past, linestyle=linestyle_past)
    
    # To plot forecast data
    ax.plot(t[future], target_points[future], label='forecast', color=color_forecast, linestyle=linestyle_forecast)
    
    # To plot confidence interval of forecast values
    if show_confidence:
        ax.fill_between(t[future][1:], lower_confidence, upper_confidence, color='crimson', alpha=0.3, label='95% confidence')

    # To get number of annotations, if 0, then no annotations
    if numElems>0:
        
        # to get number of annotations
        idx = np.round(np.linspace(0, len(target_points) - 1, numElems)).astype(int)
        
        # to loop all values to plot
        for x,y in zip(t[idx],target_points[idx]):
            
            # if number of digits too long, make it scientific annotations
            if (len(str(y)) >5) & (scientific_annotation ==True):
                label = '{:0.2e}'.format(y)
            else:
                label = y
                
            # to annotate for selected values
            ax.annotate(label, # this is the text
                         (x,y), # these are the coordinates to position the label
                         textcoords="offset points", # how to position the text
                         xytext=(1,15), # distance from text to points (x,y)
                         arrowprops=dict(facecolor='black',arrowstyle="simple"),
                         ha='center' , fontsize=10) # horizontal alignment can be left, right or center

    # setup label names and target
    ax.set_ylabel(y_label_name,fontweight='bold', fontsize=16)
    ax.set_xlabel(x_label_name,fontweight='bold', fontsize=16)
    ax.set_title(my_title,fontweight='bold', fontsize=20)
    
    plt.gcf().autofmt_xdate()
#     date_format = matplotlib.dates.DateFormatter('%b, %d %Y')
#     plt.gca().xaxis.set_major_formatter(date_format)
    plt.legend(loc=legend_position, prop={'size': legend_font_size})
    
    
    # print model name
    if show_model_name:
        ax.text(0.5, 0.95, \
                f"Model Forcast = {best_arima_model_name}", \
                horizontalalignment='center', \
                verticalalignment='top', \
                bbox=dict(boxstyle="round",
                       ec=(1., 0.5, 0.5),
                       fc=(1., 0.8, 0.8),
                       ), \
                transform = ax.transAxes, color='black')
    
    return plt.show()

def overall_vis(df, date, target, period, show_model_name=True,suggested_n_forcast=True, my_title = True, y_label_name = True, x_label_name = True, \
               color_past='dodgerblue', color_forecast='crimson',linestyle_past = 'solid', linestyle_forecast='--', \
                show_confidence=True, numElems=10, matplotlib_style = 'classic', \
                legend_position=2, fig_length=13.5, fig_height=6.5, legend_font_size=14, scientific_annotation=True, \
                zoom_last_n = True):
    
    date_dict = {'y':['year', 'years'], 'm':['month','months'], 'w':['week','weeks'], 'd': ['day', 'days']}

    
    # function 1
    df3 = extract_period(df, date, target, period)
    
    # function 2
    best_arima, best_arima_model_name = calibrate_airma(df3, target)
    
    if suggested_n_forcast == True:
        suggested_n_forcast = int(0.20*len(df3))
    if my_title == True:
        my_title = f'Forcasted {target}'
    if y_label_name == True:
        y_label_name = f'{target}'
    if x_label_name == True:
        x_label_name = f'by {date_dict[period][0]}'
    if zoom_last_n ==True:
        zoom_last_n = len(df3)
        
    # function 3 get_forecast_values(df, date, best_arima, N, target, show_info=False)
    forecast, confidence, last_date, number_forcasted_values = get_forecast_values(df3, date, best_arima, suggested_n_forcast,target, period=period,show_info=False)
    
    # function 4
    df_concat = concat_result(df3, forecast, date)
    
    # function 5
    return vis(target,df_concat,last_date,confidence,show_model_name, \
               best_arima_model_name=best_arima_model_name,my_title = my_title, y_label_name = y_label_name, x_label_name = x_label_name, \
        color_past=color_past, color_forecast=color_forecast,linestyle_past = linestyle_past, linestyle_forecast=linestyle_forecast, \
        show_confidence=show_confidence, numElems=numElems, matplotlib_style = matplotlib_style, \
        legend_position=legend_position, fig_length=fig_length, fig_height=fig_height, legend_font_size=legend_font_size, scientific_annotation=scientific_annotation, \
       zoom_last_n = zoom_last_n)

def get_data():
    d= {'Date': {0: '1/1/2014',
                    1: '1/1/2014',
                    2: '1/1/2014',
                    3: '1/1/2014',
                    4: '1/1/2014',
                    5: '1/1/2014',
                    6: '1/1/2014',
                    7: '1/2/2014',
                    8: '1/2/2014',
                    9: '1/2/2014',
                    10: '1/2/2014',
                    11: '1/2/2014',
                    12: '1/2/2014',
                    13: '1/2/2014',
                    14: '1/3/2014',
                    15: '1/3/2014',
                    16: '1/3/2014',
                    17: '1/3/2014',
                    18: '1/3/2014',
                    19: '1/3/2014',
                    20: '1/3/2014',
                    21: '1/4/2014',
                    22: '1/4/2014',
                    23: '1/4/2014',
                    24: '1/4/2014',
                    25: '1/4/2014',
                    26: '1/4/2014',
                    27: '1/4/2014',
                    28: '1/4/2014',
                    29: '1/5/2014',
                    30: '1/5/2014',
                    31: '1/5/2014',
                    32: '1/5/2014',
                    33: '1/5/2014',
                    34: '1/5/2014',
                    35: '1/5/2014',
                    36: '1/5/2014',
                    37: '1/6/2014',
                    38: '1/6/2014',
                    39: '1/6/2014',
                    40: '1/6/2014',
                    41: '1/6/2014',
                    42: '1/6/2014',
                    43: '1/6/2014',
                    44: '1/6/2014',
                    45: '1/7/2014',
                    46: '1/7/2014',
                    47: '1/7/2014',
                    48: '1/7/2014',
                    49: '1/7/2014',
                    50: '1/7/2014',
                    51: '1/7/2014',
                    52: '1/7/2014',
                    53: '1/8/2014',
                    54: '1/8/2014',
                    55: '1/8/2014',
                    56: '1/8/2014',
                    57: '1/8/2014',
                    58: '1/8/2014',
                    59: '1/8/2014',
                    60: '1/8/2014',
                    61: '1/9/2014',
                    62: '1/9/2014',
                    63: '1/9/2014',
                    64: '1/9/2014',
                    65: '1/9/2014',
                    66: '1/9/2014',
                    67: '1/9/2014',
                    68: '1/9/2014',
                    69: '1/10/2014',
                    70: '1/10/2014',
                    71: '1/10/2014',
                    72: '1/10/2014',
                    73: '1/10/2014',
                    74: '1/10/2014',
                    75: '1/10/2014',
                    76: '1/10/2014',
                    77: '1/11/2014',
                    78: '1/11/2014',
                    79: '1/11/2014',
                    80: '1/11/2014',
                    81: '1/11/2014',
                    82: '1/11/2014',
                    83: '1/11/2014',
                    84: '1/11/2014',
                    85: '1/12/2014',
                    86: '1/12/2014',
                    87: '1/12/2014',
                    88: '1/12/2014',
                    89: '1/12/2014',
                    90: '1/12/2014',
                    91: '1/12/2014',
                    92: '1/12/2014',
                    93: '1/1/2015',
                    94: '1/1/2015',
                    95: '1/1/2015',
                    96: '1/1/2015',
                    97: '1/1/2015',
                    98: '1/1/2015',
                    99: '1/1/2015',
                    100: '1/1/2015',
                    101: '1/1/2015',
                    102: '1/2/2015',
                    103: '1/2/2015',
                    104: '1/2/2015',
                    105: '1/2/2015',
                    106: '1/2/2015',
                    107: '1/2/2015',
                    108: '1/2/2015',
                    109: '1/2/2015',
                    110: '1/2/2015',
                    111: '1/2/2015',
                    112: '1/3/2015',
                    113: '1/3/2015',
                    114: '1/3/2015',
                    115: '1/3/2015',
                    116: '1/3/2015',
                    117: '1/3/2015',
                    118: '1/3/2015',
                    119: '1/3/2015',
                    120: '1/3/2015',
                    121: '1/3/2015',
                    122: '1/4/2015',
                    123: '1/4/2015',
                    124: '1/4/2015',
                    125: '1/4/2015',
                    126: '1/4/2015',
                    127: '1/4/2015',
                    128: '1/4/2015',
                    129: '1/4/2015',
                    130: '1/4/2015',
                    131: '1/4/2015',
                    132: '1/5/2015',
                    133: '1/5/2015',
                    134: '1/5/2015',
                    135: '1/5/2015',
                    136: '1/5/2015',
                    137: '1/5/2015',
                    138: '1/5/2015',
                    139: '1/5/2015',
                    140: '1/5/2015',
                    141: '1/5/2015',
                    142: '1/6/2015',
                    143: '1/6/2015',
                    144: '1/6/2015',
                    145: '1/6/2015',
                    146: '1/6/2015',
                    147: '1/6/2015',
                    148: '1/6/2015',
                    149: '1/6/2015',
                    150: '1/6/2015',
                    151: '1/6/2015',
                    152: '1/7/2015',
                    153: '1/7/2015',
                    154: '1/7/2015',
                    155: '1/7/2015',
                    156: '1/7/2015',
                    157: '1/7/2015',
                    158: '1/7/2015',
                    159: '1/7/2015',
                    160: '1/7/2015',
                    161: '1/7/2015',
                    162: '1/8/2015',
                    163: '1/8/2015',
                    164: '1/8/2015',
                    165: '1/8/2015',
                    166: '1/8/2015',
                    167: '1/8/2015',
                    168: '1/8/2015',
                    169: '1/8/2015',
                    170: '1/8/2015',
                    171: '1/8/2015',
                    172: '1/9/2015',
                    173: '1/9/2015',
                    174: '1/9/2015',
                    175: '1/9/2015',
                    176: '1/9/2015',
                    177: '1/9/2015',
                    178: '1/9/2015',
                    179: '1/9/2015',
                    180: '1/9/2015',
                    181: '1/9/2015',
                    182: '1/10/2015',
                    183: '1/10/2015',
                    184: '1/10/2015',
                    185: '1/10/2015',
                    186: '1/10/2015',
                    187: '1/10/2015',
                    188: '1/10/2015',
                    189: '1/10/2015',
                    190: '1/10/2015',
                    191: '1/10/2015',
                    192: '1/10/2015',
                    193: '1/11/2015',
                    194: '1/11/2015',
                    195: '1/11/2015',
                    196: '1/11/2015',
                    197: '1/11/2015',
                    198: '1/11/2015',
                    199: '1/11/2015',
                    200: '1/11/2015',
                    201: '1/11/2015',
                    202: '1/11/2015',
                    203: '1/11/2015',
                    204: '1/12/2015',
                    205: '1/12/2015',
                    206: '1/12/2015',
                    207: '1/12/2015',
                    208: '1/12/2015',
                    209: '1/12/2015',
                    210: '1/12/2015',
                    211: '1/12/2015',
                    212: '1/12/2015',
                    213: '1/12/2015',
                    214: '1/12/2015',
                    215: '1/1/2016',
                    216: '1/1/2016',
                    217: '1/1/2016',
                    218: '1/1/2016',
                    219: '1/1/2016',
                    220: '1/1/2016',
                    221: '1/1/2016',
                    222: '1/1/2016',
                    223: '1/1/2016',
                    224: '1/1/2016',
                    225: '1/1/2016',
                    226: '1/2/2016',
                    227: '1/2/2016',
                    228: '1/2/2016',
                    229: '1/2/2016',
                    230: '1/2/2016',
                    231: '1/2/2016',
                    232: '1/2/2016',
                    233: '1/2/2016',
                    234: '1/2/2016',
                    235: '1/2/2016',
                    236: '1/2/2016',
                    237: '1/3/2016',
                    238: '1/3/2016',
                    239: '1/3/2016',
                    240: '1/3/2016',
                    241: '1/3/2016',
                    242: '1/3/2016',
                    243: '1/3/2016',
                    244: '1/3/2016',
                    245: '1/3/2016',
                    246: '1/3/2016',
                    247: '1/4/2016',
                    248: '1/4/2016',
                    249: '1/4/2016',
                    250: '1/4/2016',
                    251: '1/4/2016',
                    252: '1/4/2016',
                    253: '1/4/2016',
                    254: '1/4/2016',
                    255: '1/4/2016',
                    256: '1/4/2016',
                    257: '1/4/2016',
                    258: '1/5/2016',
                    259: '1/5/2016',
                    260: '1/5/2016',
                    261: '1/5/2016',
                    262: '1/5/2016',
                    263: '1/5/2016',
                    264: '1/5/2016',
                    265: '1/5/2016',
                    266: '1/5/2016',
                    267: '1/5/2016',
                    268: '1/5/2016',
                    269: '1/6/2016',
                    270: '1/6/2016',
                    271: '1/6/2016',
                    272: '1/6/2016',
                    273: '1/6/2016',
                    274: '1/6/2016',
                    275: '1/6/2016',
                    276: '1/6/2016',
                    277: '1/6/2016',
                    278: '1/6/2016',
                    279: '1/6/2016',
                    280: '1/7/2016',
                    281: '1/7/2016',
                    282: '1/7/2016',
                    283: '1/7/2016',
                    284: '1/7/2016',
                    285: '1/7/2016',
                    286: '1/7/2016',
                    287: '1/7/2016',
                    288: '1/7/2016',
                    289: '1/7/2016',
                    290: '1/7/2016',
                    291: '1/8/2016',
                    292: '1/8/2016',
                    293: '1/8/2016',
                    294: '1/8/2016',
                    295: '1/8/2016',
                    296: '1/8/2016',
                    297: '1/8/2016',
                    298: '1/8/2016',
                    299: '1/8/2016',
                    300: '1/8/2016',
                    301: '1/8/2016',
                    302: '1/9/2016',
                    303: '1/9/2016',
                    304: '1/9/2016',
                    305: '1/9/2016',
                    306: '1/9/2016',
                    307: '1/9/2016',
                    308: '1/9/2016',
                    309: '1/9/2016',
                    310: '1/9/2016',
                    311: '1/9/2016',
                    312: '1/9/2016',
                    313: '1/10/2016',
                    314: '1/10/2016',
                    315: '1/10/2016',
                    316: '1/10/2016',
                    317: '1/10/2016',
                    318: '1/10/2016',
                    319: '1/10/2016',
                    320: '1/10/2016',
                    321: '1/10/2016',
                    322: '1/10/2016',
                    323: '1/10/2016',
                    324: '1/11/2016',
                    325: '1/11/2016',
                    326: '1/11/2016',
                    327: '1/11/2016',
                    328: '1/11/2016',
                    329: '1/11/2016',
                    330: '1/11/2016',
                    331: '1/11/2016',
                    332: '1/11/2016',
                    333: '1/11/2016',
                    334: '1/11/2016',
                    335: '1/12/2016',
                    336: '1/12/2016',
                    337: '1/12/2016',
                    338: '1/12/2016',
                    339: '1/12/2016',
                    340: '1/12/2016',
                    341: '1/12/2016',
                    342: '1/12/2016',
                    343: '1/12/2016',
                    344: '1/12/2016',
                    345: '1/12/2016',
                    346: '1/1/2017',
                    347: '1/1/2017',
                    348: '1/1/2017',
                    349: '1/1/2017',
                    350: '1/1/2017',
                    351: '1/1/2017',
                    352: '1/1/2017',
                    353: '1/1/2017',
                    354: '1/1/2017',
                    355: '1/1/2017',
                    356: '1/1/2017',
                    357: '1/2/2017',
                    358: '1/2/2017',
                    359: '1/2/2017',
                    360: '1/2/2017',
                    361: '1/2/2017',
                    362: '1/2/2017',
                    363: '1/2/2017',
                    364: '1/2/2017',
                    365: '1/2/2017',
                    366: '1/2/2017',
                    367: '1/2/2017',
                    368: '1/3/2017',
                    369: '1/3/2017',
                    370: '1/3/2017',
                    371: '1/3/2017',
                    372: '1/3/2017',
                    373: '1/3/2017',
                    374: '1/3/2017',
                    375: '1/3/2017',
                    376: '1/3/2017',
                    377: '1/3/2017',
                    378: '1/3/2017',
                    379: '1/4/2017',
                    380: '1/4/2017',
                    381: '1/4/2017',
                    382: '1/4/2017',
                    383: '1/4/2017',
                    384: '1/4/2017',
                    385: '1/4/2017',
                    386: '1/4/2017',
                    387: '1/4/2017',
                    388: '1/4/2017',
                    389: '1/4/2017',
                    390: '1/4/2017',
                    391: '1/5/2017',
                    392: '1/5/2017',
                    393: '1/5/2017',
                    394: '1/5/2017',
                    395: '1/5/2017',
                    396: '1/5/2017',
                    397: '1/5/2017',
                    398: '1/5/2017',
                    399: '1/5/2017',
                    400: '1/5/2017',
                    401: '1/5/2017',
                    402: '1/5/2017',
                    403: '1/6/2017',
                    404: '1/6/2017',
                    405: '1/6/2017',
                    406: '1/6/2017',
                    407: '1/6/2017',
                    408: '1/6/2017',
                    409: '1/6/2017',
                    410: '1/6/2017',
                    411: '1/6/2017',
                    412: '1/6/2017',
                    413: '1/6/2017',
                    414: '1/6/2017',
                    415: '1/7/2017',
                    416: '1/7/2017',
                    417: '1/7/2017',
                    418: '1/7/2017',
                    419: '1/7/2017',
                    420: '1/7/2017',
                    421: '1/7/2017',
                    422: '1/7/2017',
                    423: '1/7/2017',
                    424: '1/7/2017',
                    425: '1/7/2017',
                    426: '1/7/2017',
                    427: '1/8/2017',
                    428: '1/8/2017',
                    429: '1/8/2017',
                    430: '1/8/2017',
                    431: '1/8/2017',
                    432: '1/8/2017',
                    433: '1/8/2017',
                    434: '1/8/2017',
                    435: '1/8/2017',
                    436: '1/8/2017',
                    437: '1/8/2017',
                    438: '1/8/2017',
                    439: '1/9/2017',
                    440: '1/9/2017',
                    441: '1/9/2017',
                    442: '1/9/2017',
                    443: '1/9/2017',
                    444: '1/9/2017',
                    445: '1/9/2017',
                    446: '1/9/2017',
                    447: '1/9/2017',
                    448: '1/9/2017',
                    449: '1/9/2017',
                    450: '1/9/2017',
                    451: '1/10/2017',
                    452: '1/10/2017',
                    453: '1/10/2017',
                    454: '1/10/2017',
                    455: '1/10/2017',
                    456: '1/10/2017',
                    457: '1/10/2017',
                    458: '1/10/2017',
                    459: '1/10/2017',
                    460: '1/10/2017',
                    461: '1/10/2017',
                    462: '1/10/2017',
                    463: '1/11/2017',
                    464: '1/11/2017',
                    465: '1/11/2017',
                    466: '1/11/2017',
                    467: '1/11/2017',
                    468: '1/11/2017',
                    469: '1/11/2017',
                    470: '1/11/2017',
                    471: '1/11/2017',
                    472: '1/11/2017',
                    473: '1/11/2017',
                    474: '1/11/2017',
                    475: '1/12/2017',
                    476: '1/12/2017',
                    477: '1/12/2017',
                    478: '1/12/2017',
                    479: '1/12/2017',
                    480: '1/12/2017',
                    481: '1/12/2017',
                    482: '1/12/2017',
                    483: '1/12/2017',
                    484: '1/12/2017',
                    485: '1/12/2017',
                    486: '1/12/2017',
                    487: '1/1/2018',
                    488: '1/1/2018',
                    489: '1/1/2018',
                    490: '1/1/2018',
                    491: '1/1/2018',
                    492: '1/1/2018',
                    493: '1/1/2018',
                    494: '1/1/2018',
                    495: '1/1/2018',
                    496: '1/1/2018',
                    497: '1/1/2018',
                    498: '1/1/2018',
                    499: '1/2/2018',
                    500: '1/2/2018',
                    501: '1/2/2018',
                    502: '1/2/2018',
                    503: '1/2/2018',
                    504: '1/2/2018',
                    505: '1/2/2018',
                    506: '1/2/2018',
                    507: '1/2/2018',
                    508: '1/2/2018',
                    509: '1/2/2018',
                    510: '1/2/2018',
                    511: '1/3/2018',
                    512: '1/3/2018',
                    513: '1/3/2018',
                    514: '1/3/2018',
                    515: '1/3/2018',
                    516: '1/3/2018',
                    517: '1/3/2018',
                    518: '1/3/2018',
                    519: '1/3/2018',
                    520: '1/3/2018',
                    521: '1/3/2018',
                    522: '1/3/2018',
                    523: '1/4/2018',
                    524: '1/4/2018',
                    525: '1/4/2018',
                    526: '1/4/2018',
                    527: '1/4/2018',
                    528: '1/4/2018',
                    529: '1/4/2018',
                    530: '1/4/2018',
                    531: '1/4/2018',
                    532: '1/4/2018',
                    533: '1/4/2018',
                    534: '1/4/2018',
                    535: '1/5/2018',
                    536: '1/5/2018',
                    537: '1/5/2018',
                    538: '1/5/2018',
                    539: '1/5/2018',
                    540: '1/5/2018',
                    541: '1/5/2018',
                    542: '1/5/2018',
                    543: '1/5/2018',
                    544: '1/5/2018',
                    545: '1/5/2018',
                    546: '1/5/2018',
                    547: '1/6/2018',
                    548: '1/6/2018',
                    549: '1/6/2018',
                    550: '1/6/2018',
                    551: '1/6/2018',
                    552: '1/6/2018',
                    553: '1/6/2018',
                    554: '1/6/2018',
                    555: '1/6/2018',
                    556: '1/6/2018',
                    557: '1/6/2018',
                    558: '1/6/2018',
                    559: '1/7/2018',
                    560: '1/7/2018',
                    561: '1/7/2018',
                    562: '1/7/2018',
                    563: '1/7/2018',
                    564: '1/7/2018',
                    565: '1/7/2018',
                    566: '1/7/2018',
                    567: '1/7/2018',
                    568: '1/7/2018',
                    569: '1/7/2018',
                    570: '1/7/2018',
                    571: '1/8/2018',
                    572: '1/8/2018',
                    573: '1/8/2018',
                    574: '1/8/2018',
                    575: '1/8/2018',
                    576: '1/8/2018',
                    577: '1/8/2018',
                    578: '1/8/2018',
                    579: '1/8/2018',
                    580: '1/8/2018',
                    581: '1/8/2018',
                    582: '1/8/2018',
                    583: '1/9/2018',
                    584: '1/9/2018',
                    585: '1/9/2018',
                    586: '1/9/2018',
                    587: '1/9/2018',
                    588: '1/9/2018',
                    589: '1/9/2018',
                    590: '1/9/2018',
                    591: '1/9/2018',
                    592: '1/9/2018',
                    593: '1/9/2018',
                    594: '1/9/2018',
                    595: '1/10/2018',
                    596: '1/10/2018',
                    597: '1/10/2018',
                    598: '1/10/2018',
                    599: '1/10/2018',
                    600: '1/10/2018',
                    601: '1/10/2018',
                    602: '1/10/2018',
                    603: '1/10/2018',
                    604: '1/10/2018',
                    605: '1/10/2018',
                    606: '1/10/2018',
                    607: '1/11/2018',
                    608: '1/11/2018',
                    609: '1/11/2018',
                    610: '1/11/2018',
                    611: '1/11/2018',
                    612: '1/11/2018',
                    613: '1/11/2018',
                    614: '1/11/2018',
                    615: '1/11/2018',
                    616: '1/11/2018',
                    617: '1/11/2018',
                    618: '1/11/2018',
                    619: '1/12/2018',
                    620: '1/12/2018',
                    621: '1/12/2018',
                    622: '1/12/2018',
                    623: '1/12/2018',
                    624: '1/12/2018',
                    625: '1/12/2018',
                    626: '1/12/2018',
                    627: '1/12/2018',
                    628: '1/12/2018',
                    629: '1/12/2018',
                    630: '1/12/2018',
                    631: '1/1/2019',
                    632: '1/1/2019',
                    633: '1/1/2019',
                    634: '1/1/2019',
                    635: '1/1/2019',
                    636: '1/1/2019',
                    637: '1/1/2019',
                    638: '1/1/2019',
                    639: '1/1/2019',
                    640: '1/1/2019',
                    641: '1/1/2019',
                    642: '1/1/2019',
                    643: '1/2/2019',
                    644: '1/2/2019',
                    645: '1/2/2019',
                    646: '1/2/2019',
                    647: '1/2/2019',
                    648: '1/2/2019',
                    649: '1/2/2019',
                    650: '1/2/2019',
                    651: '1/2/2019',
                    652: '1/2/2019',
                    653: '1/2/2019',
                    654: '1/2/2019',
                    655: '1/3/2019',
                    656: '1/3/2019',
                    657: '1/3/2019',
                    658: '1/3/2019',
                    659: '1/3/2019',
                    660: '1/3/2019',
                    661: '1/3/2019',
                    662: '1/3/2019',
                    663: '1/3/2019',
                    664: '1/3/2019',
                    665: '1/3/2019',
                    666: '1/3/2019',
                    667: '1/4/2019',
                    668: '1/4/2019',
                    669: '1/4/2019',
                    670: '1/4/2019',
                    671: '1/4/2019',
                    672: '1/4/2019',
                    673: '1/4/2019',
                    674: '1/4/2019',
                    675: '1/4/2019',
                    676: '1/4/2019',
                    677: '1/4/2019',
                    678: '1/5/2019',
                    679: '1/5/2019',
                    680: '1/5/2019',
                    681: '1/5/2019',
                    682: '1/5/2019',
                    683: '1/5/2019',
                    684: '1/5/2019',
                    685: '1/5/2019',
                    686: '1/5/2019',
                    687: '1/5/2019',
                    688: '1/5/2019',
                    689: '1/5/2019',
                    690: '1/6/2019',
                    691: '1/6/2019',
                    692: '1/6/2019',
                    693: '1/6/2019',
                    694: '1/6/2019',
                    695: '1/6/2019',
                    696: '1/6/2019',
                    697: '1/6/2019',
                    698: '1/6/2019',
                    699: '1/6/2019',
                    700: '1/6/2019',
                    701: '1/6/2019',
                    702: '1/6/2019',
                    703: '1/7/2019',
                    704: '1/7/2019',
                    705: '1/7/2019',
                    706: '1/7/2019',
                    707: '1/7/2019',
                    708: '1/7/2019',
                    709: '1/7/2019',
                    710: '1/7/2019',
                    711: '1/7/2019',
                    712: '1/7/2019',
                    713: '1/7/2019',
                    714: '1/7/2019',
                    715: '1/8/2019',
                    716: '1/8/2019',
                    717: '1/8/2019',
                    718: '1/8/2019',
                    719: '1/8/2019',
                    720: '1/8/2019',
                    721: '1/8/2019',
                    722: '1/8/2019',
                    723: '1/8/2019',
                    724: '1/8/2019',
                    725: '1/8/2019',
                    726: '1/8/2019',
                    727: '1/8/2019',
                    728: '1/9/2019',
                    729: '1/9/2019',
                    730: '1/9/2019',
                    731: '1/9/2019',
                    732: '1/9/2019',
                    733: '1/9/2019',
                    734: '1/9/2019',
                    735: '1/9/2019',
                    736: '1/9/2019',
                    737: '1/9/2019',
                    738: '1/9/2019',
                    739: '1/9/2019',
                    740: '1/9/2019',
                    741: '1/10/2019',
                    742: '1/10/2019',
                    743: '1/10/2019',
                    744: '1/10/2019',
                    745: '1/10/2019',
                    746: '1/10/2019',
                    747: '1/10/2019',
                    748: '1/10/2019',
                    749: '1/10/2019',
                    750: '1/10/2019',
                    751: '1/10/2019',
                    752: '1/10/2019',
                    753: '1/10/2019',
                    754: '1/11/2019',
                    755: '1/11/2019',
                    756: '1/11/2019',
                    757: '1/11/2019',
                    758: '1/11/2019',
                    759: '1/11/2019',
                    760: '1/11/2019',
                    761: '1/11/2019',
                    762: '1/11/2019',
                    763: '1/11/2019',
                    764: '1/11/2019',
                    765: '1/11/2019',
                    766: '1/12/2019',
                    767: '1/12/2019',
                    768: '1/12/2019',
                    769: '1/12/2019',
                    770: '1/12/2019',
                    771: '1/12/2019',
                    772: '1/12/2019',
                    773: '1/12/2019',
                    774: '1/12/2019',
                    775: '1/12/2019',
                    776: '1/12/2019',
                    777: '1/12/2019',
                    778: '1/12/2019',
                    779: '1/1/2020',
                    780: '1/1/2020',
                    781: '1/1/2020',
                    782: '1/1/2020',
                    783: '1/1/2020',
                    784: '1/1/2020',
                    785: '1/1/2020',
                    786: '1/1/2020',
                    787: '1/1/2020',
                    788: '1/1/2020',
                    789: '1/1/2020',
                    790: '1/1/2020',
                    791: '1/1/2020',
                    792: '1/2/2020',
                    793: '1/2/2020',
                    794: '1/2/2020',
                    795: '1/2/2020',
                    796: '1/2/2020',
                    797: '1/2/2020',
                    798: '1/2/2020',
                    799: '1/2/2020',
                    800: '1/2/2020',
                    801: '1/2/2020',
                    802: '1/2/2020',
                    803: '1/2/2020',
                    804: '1/2/2020',
                    805: '1/3/2020',
                    806: '1/3/2020',
                    807: '1/3/2020',
                    808: '1/3/2020',
                    809: '1/3/2020',
                    810: '1/3/2020',
                    811: '1/3/2020',
                    812: '1/3/2020',
                    813: '1/3/2020',
                    814: '1/3/2020',
                    815: '1/3/2020',
                    816: '1/3/2020',
                    817: '1/3/2020',
                    818: '1/4/2020',
                    819: '1/4/2020',
                    820: '1/4/2020',
                    821: '1/4/2020',
                    822: '1/4/2020',
                    823: '1/4/2020',
                    824: '1/4/2020',
                    825: '1/4/2020',
                    826: '1/4/2020',
                    827: '1/4/2020',
                    828: '1/4/2020',
                    829: '1/4/2020',
                    830: '1/4/2020',
                    831: '1/5/2020',
                    832: '1/5/2020',
                    833: '1/5/2020',
                    834: '1/5/2020',
                    835: '1/5/2020',
                    836: '1/5/2020',
                    837: '1/5/2020',
                    838: '1/5/2020',
                    839: '1/5/2020',
                    840: '1/5/2020',
                    841: '1/5/2020',
                    842: '1/5/2020',
                    843: '1/5/2020',
                    844: '1/6/2020',
                    845: '1/6/2020',
                    846: '1/6/2020',
                    847: '1/6/2020',
                    848: '1/6/2020',
                    849: '1/6/2020',
                    850: '1/6/2020',
                    851: '1/6/2020',
                    852: '1/6/2020',
                    853: '1/6/2020',
                    854: '1/6/2020',
                    855: '1/6/2020',
                    856: '1/6/2020',
                    857: '1/7/2020',
                    858: '1/7/2020',
                    859: '1/7/2020',
                    860: '1/7/2020',
                    861: '1/7/2020',
                    862: '1/7/2020',
                    863: '1/7/2020',
                    864: '1/7/2020',
                    865: '1/7/2020',
                    866: '1/7/2020',
                    867: '1/7/2020',
                    868: '1/7/2020',
                    869: '1/7/2020',
                    870: '1/8/2020',
                    871: '1/8/2020',
                    872: '1/8/2020',
                    873: '1/8/2020',
                    874: '1/8/2020',
                    875: '1/8/2020',
                    876: '1/8/2020',
                    877: '1/8/2020',
                    878: '1/8/2020',
                    879: '1/8/2020',
                    880: '1/8/2020',
                    881: '1/8/2020',
                    882: '1/8/2020',
                    883: '1/9/2020',
                    884: '1/9/2020',
                    885: '1/9/2020',
                    886: '1/9/2020',
                    887: '1/9/2020',
                    888: '1/9/2020',
                    889: '1/9/2020',
                    890: '1/9/2020',
                    891: '1/9/2020',
                    892: '1/9/2020',
                    893: '1/9/2020',
                    894: '1/9/2020',
                    895: '1/9/2020',
                    896: '1/10/2020',
                    897: '1/10/2020',
                    898: '1/10/2020',
                    899: '1/10/2020',
                    900: '1/10/2020',
                    901: '1/10/2020',
                    902: '1/10/2020',
                    903: '1/10/2020',
                    904: '1/10/2020',
                    905: '1/10/2020',
                    906: '1/10/2020',
                    907: '1/10/2020',
                    908: '1/10/2020',
                    909: '1/11/2020',
                    910: '1/11/2020',
                    911: '1/11/2020',
                    912: '1/11/2020',
                    913: '1/11/2020',
                    914: '1/11/2020',
                    915: '1/11/2020',
                    916: '1/11/2020',
                    917: '1/11/2020',
                    918: '1/11/2020',
                    919: '1/11/2020',
                    920: '1/11/2020',
                    921: '1/11/2020',
                    922: '1/12/2020',
                    923: '1/12/2020',
                    924: '1/12/2020',
                    925: '1/12/2020',
                    926: '1/12/2020',
                    927: '1/12/2020',
                    928: '1/12/2020',
                    929: '1/12/2020',
                    930: '1/12/2020',
                    931: '1/12/2020',
                    932: '1/12/2020',
                    933: '1/12/2020',
                    934: '1/12/2020'},
                    'Volume': {0: 1346973.89,
                    1: 677825.87,
                    2: 1210358.5,
                    3: 436760.72,
                    4: 433.99,
                    5: 1229415.43,
                    6: 790802.06,
                    7: 1058681.45,
                    8: 442604.98,
                    9: 829241.95,
                    10: 205538.32,
                    11: 699.1,
                    12: 885082.1,
                    13: 507910.17,
                    14: 1089614.89,
                    15: 550634.44,
                    16: 1100553.11,
                    17: 262253.79,
                    18: 1421.69,
                    19: 1191730.78,
                    20: 612966.58,
                    21: 1073261.19,
                    22: 278106.1556,
                    23: 665210.87,
                    24: 1316461.38,
                    25: 351818.36,
                    26: 1758.77,
                    27: 1357049.52,
                    28: 879226.26,
                    29: 1289022.39,
                    30: 389241.7797,
                    31: 811401.72,
                    32: 1515828.53,
                    33: 309505.04,
                    34: 2275.23,
                    35: 1713073.07,
                    36: 1041092.33,
                    37: 1491952.97,
                    38: 501514.3199,
                    39: 915308.07,
                    40: 1731305.68,
                    41: 339498.79,
                    42: 2894.75,
                    43: 1887919.38,
                    44: 1292277.97,
                    45: 1493795.1,
                    46: 480080.693,
                    47: 908803.0,
                    48: 1851818.65,
                    49: 398613.29,
                    50: 3980.05,
                    51: 2075215.48,
                    52: 1608205.35,
                    53: 1466581.08,
                    54: 481474.2739,
                    55: 852560.37,
                    56: 1572367.61,
                    57: 359737.22,
                    58: 4809.61,
                    59: 1838571.12,
                    60: 1392830.13,
                    61: 1454892.43,
                    62: 353572.4571,
                    63: 730446.72,
                    64: 1192245.73,
                    65: 253182.77,
                    66: 9177.55,
                    67: 1315307.39,
                    68: 1042745.01,
                    69: 1273274.32,
                    70: 251980.2414,
                    71: 529549.84,
                    72: 757297.45,
                    73: 147186.47,
                    74: 9491.83,
                    75: 952823.48,
                    76: 772633.63,
                    77: 1263674.68,
                    78: 307163.1991,
                    79: 435708.54,
                    80: 721556.47,
                    81: 115297.91,
                    82: 3459.93,
                    83: 664778.29,
                    84: 477091.11,
                    85: 1056909.76,
                    86: 277088.4483,
                    87: 402694.34,
                    88: 635010.84,
                    89: 240868.72,
                    90: 11212.42,
                    91: 591159.27,
                    92: 252554.77,
                    93: 1739132.256,
                    94: 224316.1444,
                    95: 391341.3591,
                    96: 732333.494,
                    97: 1172744.589,
                    98: 563134.992,
                    99: 20223.811,
                    100: 962854.6258,
                    101: 772531.375,
                    102: 1173442.578,
                    103: 140170.5023,
                    104: 304945.3773,
                    105: 495118.4596,
                    106: 853244.4689,
                    107: 315289.152,
                    108: 24154.962,
                    109: 18.72,
                    110: 811585.251,
                    111: 422913.373,
                    112: 1350054.484,
                    113: 310055.9878,
                    114: 294898.4348,
                    115: 653160.2256,
                    116: 1075764.182,
                    117: 346540.5598,
                    118: 29221.012,
                    119: 0.0,
                    120: 1034510.996,
                    121: 589796.942,
                    122: 1366797.473,
                    123: 187567.7294,
                    124: 302631.6236,
                    125: 675694.0762,
                    126: 1183960.098,
                    127: 404588.0856,
                    128: 30111.185,
                    129: 12.48,
                    130: 1173989.204,
                    131: 835952.082,
                    132: 1591647.124,
                    133: 288397.279,
                    134: 407394.2303,
                    135: 825798.5171,
                    136: 1447323.553,
                    137: 436667.0618,
                    138: 42928.078,
                    139: 7.488,
                    140: 1502251.718,
                    141: 946044.294,
                    142: 1769400.25,
                    143: 347603.3272,
                    144: 426876.7607,
                    145: 890423.0059,
                    146: 1520061.771,
                    147: 479738.0732,
                    148: 41879.512,
                    149: 355.054,
                    150: 1682692.493,
                    151: 1179209.367,
                    152: 1785115.571,
                    153: 260847.2152,
                    154: 488829.2318,
                    155: 915785.0744,
                    156: 1674342.238,
                    157: 510092.5406,
                    158: 33500.827,
                    159: 1365.248,
                    160: 1731877.996,
                    161: 1298965.539,
                    162: 1734861.135,
                    163: 273649.1857,
                    164: 466409.9638,
                    165: 873802.4516,
                    166: 1581172.776,
                    167: 461139.0069,
                    168: 24294.18,
                    169: 631.764,
                    170: 1753674.453,
                    171: 1091495.801,
                    172: 1749241.579,
                    173: 166737.35,
                    174: 338392.9177,
                    175: 735689.9688,
                    176: 1180488.913,
                    177: 382897.2824,
                    178: 20306.839,
                    179: 624.015,
                    180: 1214508.787,
                    181: 778871.166,
                    182: 1447929.09,
                    183: 89291.16196,
                    184: 265225.7605,
                    185: 521611.4124,
                    186: 819404.7769,
                    187: 218970.7971,
                    188: 27597.73764,
                    189: 1262.722,
                    190: 875910.1697,
                    191: 688643.117,
                    192: 28.924,
                    193: 1393531.365,
                    194: 31629.35776,
                    195: 254493.845,
                    196: 419062.1167,
                    197: 663024.4301,
                    198: 207952.1267,
                    199: 28021.464,
                    200: 1711.974,
                    201: 527776.1568,
                    202: 320148.363,
                    203: 17.83,
                    204: 1229172.449,
                    205: 20226.99796,
                    206: 241447.699,
                    207: 391826.6264,
                    208: 613604.8239,
                    209: 238474.7363,
                    210: 28805.91276,
                    211: 9019.903,
                    212: 489076.228,
                    213: 182696.779,
                    214: 6.293,
                    215: 1839414.84,
                    216: 192524.7132,
                    217: 430439.517,
                    218: 754534.2777,
                    219: 1341074.526,
                    220: 636580.3392,
                    221: 33926.26,
                    222: 19827.14924,
                    223: 1048312.105,
                    224: 741616.389,
                    225: 17.885,
                    226: 1094560.82,
                    227: 166363.2043,
                    228: 263007.374,
                    229: 448055.3929,
                    230: 850956.7494,
                    231: 280497.277,
                    232: 40970.3694,
                    233: 5673.08536,
                    234: 647800.0775,
                    235: 337136.714,
                    236: 9.936,
                    237: 1273194.988,
                    238: 334339.0123,
                    239: 277081.19,
                    240: 605301.9533,
                    241: 1062971.828,
                    242: 361979.4882,
                    243: 44524.75692,
                    244: 11065.4464,
                    245: 909575.2036,
                    246: 551379.133,
                    247: 1361665.149,
                    248: 182070.0,
                    249: 278088.807,
                    250: 109.44,
                    251: 634592.14,
                    252: 1167166.953,
                    253: 454540.278,
                    254: 54463.6116,
                    255: 4439.382,
                    256: 1015892.787,
                    257: 578555.452,
                    258: 1788499.457,
                    259: 220784.2598,
                    260: 351247.476,
                    261: 421.14,
                    262: 838263.3561,
                    263: 1411169.588,
                    264: 432452.3106,
                    265: 44903.12028,
                    266: 3273.11424,
                    267: 1279351.7,
                    268: 930643.478,
                    269: 2000922.797,
                    270: 288103.5862,
                    271: 410341.273,
                    272: 1839.096,
                    273: 958121.9468,
                    274: 1553591.683,
                    275: 494009.8998,
                    276: 52468.75196,
                    277: 3460.466,
                    278: 1556195.763,
                    279: 1132426.738,
                    280: 1866309.173,
                    281: 278869.379,
                    282: 394297.008,
                    283: 3978.041,
                    284: 908749.99,
                    285: 1654797.934,
                    286: 521942.679,
                    287: 63674.0098,
                    288: 3237.041,
                    289: 1610425.044,
                    290: 1258269.701,
                    291: 1924014.773,
                    292: 348002.72,
                    293: 430169.631,
                    294: 865.14,
                    295: 875246.511,
                    296: 1587116.581,
                    297: 504320.715,
                    298: 68598.3768,
                    299: 2887.288,
                    300: 1740011.111,
                    301: 1111071.153,
                    302: 1865360.952,
                    303: 201088.871,
                    304: 323766.335,
                    305: 9039.672,
                    306: 712233.02,
                    307: 1169895.526,
                    308: 391162.729,
                    309: 64881.8488,
                    310: 3242.256,
                    311: 1259683.463,
                    312: 733171.327,
                    313: 1399595.449,
                    314: 72014.232,
                    315: 233125.794,
                    316: 5654.268,
                    317: 516456.994,
                    318: 769424.324,
                    319: 225548.494,
                    320: 65035.5324,
                    321: 2475.787,
                    322: 725598.4156,
                    323: 595759.71,
                    324: 1403599.075,
                    325: 48788.352,
                    326: 208780.298,
                    327: 5024.724,
                    328: 409178.454,
                    329: 638755.685,
                    330: 287035.884,
                    331: 49476.2958,
                    332: 1519.875,
                    333: 485309.7372,
                    334: 282784.62,
                    335: 1082858.577,
                    336: 35182.326,
                    337: 208562.54,
                    338: 7219.08,
                    339: 374655.939,
                    340: 639738.583,
                    341: 248429.634,
                    342: 27170.2538,
                    343: 1256.572,
                    344: 502585.5332,
                    345: 159972.962,
                    346: 2250663.319,
                    347: 218102.915,
                    348: 413930.865,
                    349: 17617.538,
                    350: 823996.612,
                    351: 1414352.614,
                    352: 615601.699,
                    353: 96880.901,
                    354: 1595.018,
                    355: 1073141.97,
                    356: 797743.564,
                    357: 1238240.413,
                    358: 195173.665,
                    359: 237863.663,
                    360: 19515.144,
                    361: 531714.383,
                    362: 780185.763,
                    363: 260048.68,
                    364: 74219.9638,
                    365: 825.258,
                    366: 620767.2335,
                    367: 303495.046,
                    368: 1473500.64,
                    369: 296457.051,
                    370: 268431.725,
                    371: 20729.352,
                    372: 582760.271,
                    373: 1008493.234,
                    374: 338426.651,
                    375: 64308.5992,
                    376: 770.91,
                    377: 874262.7771,
                    378: 526506.47,
                    379: 1577683.964,
                    380: 229472.093,
                    381: 259833.824,
                    382: 21660.258,
                    383: 621022.306,
                    384: 1100276.823,
                    385: 438945.805,
                    386: 75832.935,
                    387: 389.419,
                    388: 946130.9679,
                    389: 583760.605,
                    390: 491.832,
                    391: 1981965.528,
                    392: 253944.928,
                    393: 312614.669,
                    394: 29592.804,
                    395: 828751.093,
                    396: 1277580.587,
                    397: 453081.425,
                    398: 85748.6232,
                    399: 245.361,
                    400: 1285375.837,
                    401: 896343.475,
                    402: 7647.473,
                    403: 2149660.034,
                    404: 347982.339,
                    405: 342424.953,
                    406: 57014.364,
                    407: 919852.83,
                    408: 1425535.221,
                    409: 575708.976,
                    410: 116531.579,
                    411: 206.995,
                    412: 1475423.76,
                    413: 1002472.553,
                    414: 8734.968,
                    415: 1960839.594,
                    416: 354995.815,
                    417: 416788.627,
                    418: 52639.669,
                    419: 917099.18,
                    420: 1597695.993,
                    421: 562621.982,
                    422: 117176.1116,
                    423: 172.626,
                    424: 1692018.94,
                    425: 1216408.324,
                    426: 7941.288,
                    427: 1820630.918,
                    428: 323096.702,
                    429: 358207.407,
                    430: 57231.889,
                    431: 915701.431,
                    432: 1508837.162,
                    433: 535622.897,
                    434: 124567.326,
                    435: 24.567,
                    436: 1595344.864,
                    437: 1123220.087,
                    438: 7291.745,
                    439: 1812374.868,
                    440: 158353.188,
                    441: 296192.867,
                    442: 15150.598,
                    443: 778111.051,
                    444: 1082339.396,
                    445: 477350.273,
                    446: 132522.9116,
                    447: 4.902,
                    448: 1113870.968,
                    449: 715541.925,
                    450: 2627.872,
                    451: 1294904.866,
                    452: 79703.967,
                    453: 211282.175,
                    454: 10220.95,
                    455: 565154.424,
                    456: 682966.378,
                    457: 278376.236,
                    458: 105537.4702,
                    459: 0.264,
                    460: 651159.7756,
                    461: 558042.357,
                    462: 2834.868,
                    463: 1354468.321,
                    464: 72560.851,
                    465: 197163.69,
                    466: 6668.198,
                    467: 441573.252,
                    468: 592716.5186,
                    469: 226338.677,
                    470: 114458.914,
                    471: 0.0,
                    472: 504512.0292,
                    473: 288278.88,
                    474: 1559.748,
                    475: 1072255.96,
                    476: 94814.664,
                    477: 181562.676,
                    478: 5907.034,
                    479: 386804.601,
                    480: 592654.218,
                    481: 258690.282,
                    482: 90683.725,
                    483: 0.0,
                    484: 419109.8335,
                    485: 144634.12,
                    486: 1653.22,
                    487: 0.51,
                    488: 2087574.134,
                    489: 210471.219,
                    490: 415046.717,
                    491: 11015.108,
                    492: 716737.47,
                    493: 1257018.036,
                    494: 598423.345,
                    495: 241068.513,
                    496: 0.0,
                    497: 949744.0946,
                    498: 766732.762,
                    499: 0.0,
                    500: 1349173.952,
                    501: 207804.28,
                    502: 230398.074,
                    503: 12394.363,
                    504: 552585.394,
                    505: 863668.175,
                    506: 361311.028,
                    507: 139186.498,
                    508: 0.0,
                    509: 678854.2447,
                    510: 260966.616,
                    511: 0.085,
                    512: 1410440.008,
                    513: 390593.743,
                    514: 234601.503,
                    515: 21816.103,
                    516: 709207.593,
                    517: 1028404.108,
                    518: 493973.36,
                    519: 116007.61,
                    520: 0.0,
                    521: 856734.4448,
                    522: 546141.148,
                    523: 0.0,
                    524: 1601445.613,
                    525: 240677.248,
                    526: 232700.328,
                    527: 27469.145,
                    528: 656917.559,
                    529: 953594.701,
                    530: 486478.558,
                    531: 154445.821,
                    532: 0.0,
                    533: 997163.793,
                    534: 601703.856,
                    535: 0.0,
                    536: 2037693.524,
                    537: 315272.837,
                    538: 287330.847,
                    539: 30052.713,
                    540: 880830.267,
                    541: 1283126.192,
                    542: 619096.916,
                    543: 195014.289,
                    544: 0.0,
                    545: 1300243.782,
                    546: 893055.415,
                    547: 0.0,
                    548: 2113647.364,
                    549: 360934.557,
                    550: 319683.805,
                    551: 32434.513,
                    552: 977582.505,
                    553: 1387837.316,
                    554: 624608.245,
                    555: 207396.102,
                    556: 0.0,
                    557: 1488795.149,
                    558: 1066650.598,
                    559: 0.0,
                    560: 2097971.055,
                    561: 348328.557,
                    562: 349584.012,
                    563: 26463.781,
                    564: 916569.87,
                    565: 1501496.279,
                    566: 660547.162,
                    567: 208058.069,
                    568: 0.0,
                    569: 1646690.415,
                    570: 1229809.862,
                    571: 0.0,
                    572: 2002439.995,
                    573: 277324.01,
                    574: 353476.092,
                    575: 17406.587,
                    576: 939838.742,
                    577: 1385877.915,
                    578: 642793.018,
                    579: 186219.859,
                    580: 0.0,
                    581: 1600627.975,
                    582: 1134228.67,
                    583: 0.0,
                    584: 1832159.673,
                    585: 187283.807,
                    586: 261888.502,
                    587: 12894.296,
                    588: 757610.733,
                    589: 1001459.525,
                    590: 511392.747,
                    591: 183488.34,
                    592: 0.0,
                    593: 1118999.299,
                    594: 704289.018,
                    595: 0.0,
                    596: 1416419.984,
                    597: 108691.434,
                    598: 217531.081,
                    599: 13768.021,
                    600: 520876.486,
                    601: 675600.343,
                    602: 301755.207,
                    603: 149473.075,
                    604: 0.0,
                    605: 674116.6809,
                    606: 491624.691,
                    607: 0.0,
                    608: 1499260.411,
                    609: 74708.844,
                    610: 212104.531,
                    611: 10957.031,
                    612: 515093.033,
                    613: 643078.704,
                    614: 298338.857,
                    615: 120814.883,
                    616: 0.0,
                    617: 532540.0086,
                    618: 291209.205,
                    619: 0.0,
                    620: 1366062.698,
                    621: 112139.427,
                    622: 146083.299,
                    623: 7855.889,
                    624: 425325.724,
                    625: 484846.816,
                    626: 272271.088,
                    627: 172534.224,
                    628: 0.0,
                    629: 378220.8545,
                    630: 90095.119,
                    631: 0.0,
                    632: 2250196.729,
                    633: 229450.778,
                    634: 460009.93,
                    635: 11615.656,
                    636: 722283.643,
                    637: 1324336.531,
                    638: 635324.88,
                    639: 248145.194,
                    640: 0.0,
                    641: 998103.6949,
                    642: 721910.964,
                    643: 0.0,
                    644: 1212230.184,
                    645: 163038.348,
                    646: 187409.787,
                    647: 3572.161,
                    648: 459015.042,
                    649: 775095.079,
                    650: 303967.601,
                    651: 177643.972,
                    652: 0.0,
                    653: 518529.148,
                    654: 273342.678,
                    655: 0.0,
                    656: 1586541.991,
                    657: 348558.825,
                    658: 195210.3,
                    659: 8745.495,
                    660: 747099.465,
                    661: 921035.65,
                    662: 472454.178,
                    663: 171239.614,
                    664: 0.0,
                    665: 868883.6108,
                    666: 520491.871,
                    667: 0.0,
                    668: 1526338.829,
                    669: 260400.101,
                    670: 221721.919,
                    671: 10139.878,
                    672: 688260.501,
                    673: 922657.255,
                    674: 475736.555,
                    675: 180242.103,
                    676: 962309.0969,
                    677: 638785.762,
                    678: 0.0,
                    679: 2070232.414,
                    680: 213822.335,
                    681: 294203.131,
                    682: 12697.693,
                    683: 929865.85,
                    684: 1234498.063,
                    685: 614734.931,
                    686: 252367.075,
                    687: 0.0,
                    688: 1253126.346,
                    689: 890365.162,
                    690: 112636.2768,
                    691: 0.0,
                    692: 2367853.079,
                    693: 366875.972,
                    694: 335780.899,
                    695: 14230.154,
                    696: 974322.636,
                    697: 1358146.112,
                    698: 689746.683,
                    699: 313565.737,
                    700: 0.0,
                    701: 1407937.443,
                    702: 1017321.008,
                    703: 113534.8656,
                    704: 1.2,
                    705: 1748440.507,
                    706: 260280.083,
                    707: 276484.071,
                    708: 10268.906,
                    709: 891809.766,
                    710: 1220492.595,
                    711: 604562.798,
                    712: 238405.536,
                    713: 1389678.196,
                    714: 1041501.477,
                    715: 110929.5144,
                    716: 0.0,
                    717: 1965172.455,
                    718: 272938.315,
                    719: 339574.581,
                    720: 11116.213,
                    721: 994187.864,
                    722: 1448855.589,
                    723: 701550.251,
                    724: 247246.095,
                    725: 0.0,
                    726: 1557028.372,
                    727: 1140412.751,
                    728: 115339.7352,
                    729: 0.0,
                    730: 1677321.591,
                    731: 109164.532,
                    732: 250298.411,
                    733: 6033.783,
                    734: 775039.909,
                    735: 1022205.832,
                    736: 536874.218,
                    737: 200866.341,
                    738: 0.0,
                    739: 1066922.334,
                    740: 676586.313,
                    741: 87375.0864,
                    742: 0.0,
                    743: 1228504.185,
                    744: 113168.6,
                    745: 169547.098,
                    746: 6343.267,
                    747: 537122.121,
                    748: 639673.773,
                    749: 335825.872,
                    750: 180861.815,
                    751: 0.0,
                    752: 646913.7771,
                    753: 478536.406,
                    754: 85272.8256,
                    755: 0.0,
                    756: 1187562.349,
                    757: 80484.548,
                    758: 192312.654,
                    759: 4592.424,
                    760: 506995.125,
                    761: 585296.153,
                    762: 297040.846,
                    763: 133170.075,
                    764: 483405.2909,
                    765: 276454.71,
                    766: 76584.96,
                    767: 0.0,
                    768: 1196690.868,
                    769: 71646.921,
                    770: 156572.599,
                    771: 4723.552,
                    772: 338688.801,
                    773: 432346.8,
                    774: 275546.101,
                    775: 144557.785,
                    776: 0.0,
                    777: 386702.0253,
                    778: 109627.588,
                    779: 95484.3312,
                    780: 0.0,
                    781: 2165093.461,
                    782: 214448.695,
                    783: 312275.419,
                    784: 4217.227,
                    785: 623220.09,
                    786: 1111043.812,
                    787: 535131.567,
                    788: 291691.672,
                    789: 0.0,
                    790: 879890.8225,
                    791: 679369.234,
                    792: 29548.4472,
                    793: 0.0,
                    794: 174966.3407,
                    795: 12374.29,
                    796: 38114.312,
                    797: 832.56,
                    798: 46205.546,
                    799: 74469.59,
                    800: 55893.981,
                    801: 28203.786,
                    802: 0.0,
                    803: 59803.03315,
                    804: 7675.114,
                    805: 0.0,
                    806: 0.0,
                    807: 0.0,
                    808: 0.0,
                    809: 0.0,
                    810: 0.0,
                    811: 0.0,
                    812: 0.0,
                    813: 0.0,
                    814: 0.0,
                    815: 0.0,
                    816: 0.0,
                    817: 0.0,
                    818: 0.0,
                    819: 0.0,
                    820: 0.0,
                    821: 0.0,
                    822: 0.0,
                    823: 0.0,
                    824: 0.0,
                    825: 0.0,
                    826: 0.0,
                    827: 0.0,
                    828: 0.0,
                    829: 0.0,
                    830: 0.0,
                    831: 0.0,
                    832: 0.0,
                    833: 0.0,
                    834: 0.0,
                    835: 0.0,
                    836: 0.0,
                    837: 0.0,
                    838: 0.0,
                    839: 0.0,
                    840: 0.0,
                    841: 0.0,
                    842: 0.0,
                    843: 0.0,
                    844: 0.0,
                    845: 0.0,
                    846: 0.0,
                    847: 0.0,
                    848: 0.0,
                    849: 0.0,
                    850: 0.0,
                    851: 0.0,
                    852: 0.0,
                    853: 0.0,
                    854: 0.0,
                    855: 0.0,
                    856: 0.0,
                    857: 0.0,
                    858: 0.0,
                    859: 0.0,
                    860: 0.0,
                    861: 0.0,
                    862: 0.0,
                    863: 0.0,
                    864: 0.0,
                    865: 0.0,
                    866: 0.0,
                    867: 0.0,
                    868: 0.0,
                    869: 0.0,
                    870: 0.0,
                    871: 0.0,
                    872: 0.0,
                    873: 0.0,
                    874: 0.0,
                    875: 0.0,
                    876: 0.0,
                    877: 0.0,
                    878: 0.0,
                    879: 0.0,
                    880: 0.0,
                    881: 0.0,
                    882: 0.0,
                    883: 0.0,
                    884: 0.0,
                    885: 0.0,
                    886: 0.0,
                    887: 0.0,
                    888: 0.0,
                    889: 0.0,
                    890: 0.0,
                    891: 0.0,
                    892: 0.0,
                    893: 0.0,
                    894: 0.0,
                    895: 0.0,
                    896: 0.0,
                    897: 0.0,
                    898: 0.0,
                    899: 0.0,
                    900: 0.0,
                    901: 0.0,
                    902: 0.0,
                    903: 0.0,
                    904: 0.0,
                    905: 0.0,
                    906: 0.0,
                    907: 0.0,
                    908: 0.0,
                    909: 0.0,
                    910: 0.0,
                    911: 0.0,
                    912: 0.0,
                    913: 0.0,
                    914: 0.0,
                    915: 0.0,
                    916: 0.0,
                    917: 0.0,
                    918: 0.0,
                    919: 0.0,
                    920: 0.0,
                    921: 0.0,
                    922: 0.0,
                    923: 0.0,
                    924: 0.0,
                    925: 0.0,
                    926: 0.0,
                    927: 0.0,
                    928: 0.0,
                    929: 0.0,
                    930: 0.0,
                    931: 0.0,
                    932: 0.0,
                    933: 0.0,
                    934: 0.0}}
    return pd.DataFrame.from_dict(d)
